package com.server.alb.configs

import com.server.alb.services.UserDetailsServiceImpl
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.security.authentication.dao.DaoAuthenticationProvider
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder
import org.springframework.security.config.annotation.web.builders.HttpSecurity
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity
import org.springframework.security.core.userdetails.UserDetailsService
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder
import org.springframework.security.web.SecurityFilterChain


@Configuration
@EnableWebSecurity
class WebSecurityConfig {

    @Bean
    @Throws(Exception::class)
    fun securityFilterChain(http: HttpSecurity): SecurityFilterChain {
        http.authorizeHttpRequests { requests ->
            requests.requestMatchers("/").hasAnyAuthority("QC", "PROGRAMMER", "ADMIN")
            requests.requestMatchers("/inPro").hasAnyAuthority("QC", "PROGRAMMER", "ADMIN")
            requests.requestMatchers("/new").hasAnyAuthority("ADMIN", "QC")
            requests.requestMatchers("/edit/**").hasAnyAuthority("ADMIN", "QC", "PROGRAMMER")
            requests.requestMatchers("/delete/**").hasAnyAuthority("ADMIN", "QC").anyRequest().authenticated()
        }.formLogin { form ->
            form.loginPage("/login").permitAll().defaultSuccessUrl("/")
        }.logout { logout ->
            logout.permitAll()
        }.exceptionHandling { customizer ->
            customizer.accessDeniedPage("/403")
        }
        return http.build()
    }

    @Bean
    fun passwordEncoder(): BCryptPasswordEncoder {
        return BCryptPasswordEncoder()
    }

    @Bean
    fun userDetailsService(): UserDetailsService {
        return UserDetailsServiceImpl()
    }

    @Bean
    fun authenticationProvider(): DaoAuthenticationProvider {
        val authProvider = DaoAuthenticationProvider()
        authProvider.setUserDetailsService(userDetailsService())
        authProvider.setPasswordEncoder(passwordEncoder())
        return authProvider
    }

    @Throws(Exception::class)
    protected fun configure(auth: AuthenticationManagerBuilder) {
        auth.authenticationProvider(authenticationProvider())
    }

}