package com.server.alb.controllers

import com.server.alb.models.*
import com.server.alb.repositories.ListBugRepository
import com.server.alb.repositories.PlatformRepository
import com.server.alb.repositories.ProjectRepository
import com.server.alb.repositories.UserRepository
import com.server.alb.services.BugService
import com.server.alb.services.BugValidationService.Companion.isValid
import com.server.alb.services.PlatformService
import com.server.alb.services.UserService
import jakarta.servlet.http.HttpServletRequest
import jakarta.validation.Valid
import org.springframework.security.authentication.AnonymousAuthenticationToken
import org.springframework.security.core.Authentication
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.stereotype.Controller
import org.springframework.ui.Model
import org.springframework.validation.BindingResult
import org.springframework.web.bind.annotation.*
import org.springframework.web.servlet.ModelAndView
import org.springframework.web.servlet.mvc.support.RedirectAttributes


@Controller
@RequestMapping
class BugController(
    private val userRepository: UserRepository,
    private val listBugRepository: ListBugRepository,
    private val projectRepository: ProjectRepository,
    private val platformRepository: PlatformRepository,
    private val service: BugService,
    private val serUser: UserService,
    private val serPlatform: PlatformService
) {
    val qcResultType = Status.QCResultType.values()
    val worList = Status.WorkedType.values()
    val wStList = Status.WorkedStatusType.values()
    val dStList = Status.DevStatusType.values()
    val project: MutableIterable<Project> = this.projectRepository.findAll()
    var proj1: String = "1"
    var platform: MutableIterable<Platform> = this.platformRepository.findAll()
    val user: MutableIterable<User> = this.userRepository.findAll()

    @GetMapping(value = ["/login"])
    fun showLoginForm(model: Model): String {
        val authentication: Authentication? = SecurityContextHolder.getContext().authentication
        return if (authentication == null || authentication is AnonymousAuthenticationToken) {
            "login"
        } else "redirect:/inPro"
    }

    @RequestMapping(value = ["/"])
    fun viewHomePage(model: Model, pForm: String? = "", programmer: String? = "", proj: String? = proj1): String {
        val listBugs = this.listBugRepository.done(pForm ?: "", programmer ?: "", proj ?: proj1).sortedBy { it.date }
        model.addAttribute("project", project)
        model.addAttribute("platform", platform)
        model.addAttribute("user", user)
        model.addAttribute("listBugs", listBugs)
        return "index"
    }

    @RequestMapping(value = ["/inPro"])
    fun viewInProgress(model: Model, pForm: String? = "", programmer: String? = "", proj: String? = proj1): String {
        val listBugs = this.listBugRepository.inPro(pForm ?: "", programmer ?: "", proj ?: proj1).sortedBy { it.date }
        model.addAttribute("project", project)
        model.addAttribute("platform", platform)
        model.addAttribute("user", user)
        model.addAttribute("listBugs", listBugs)
        return "inPro"
    }

    @GetMapping(value = ["/new"])
    fun showNewBugForm(model: Model, proj: String? = proj1): String {
        val bug = ListBug()
        model.addAttribute("bug", bug)
        model.addAttribute("user", user)
        model.addAttribute("platform", platform)
        model.addAttribute("qcResultType", qcResultType)
        return "add_bug"
    }

    @PostMapping(value = ["/new"])
    fun submitNewBugForm(
        @Valid @ModelAttribute("bug") bug: ListBug,
        bindingResult: BindingResult,
        redirectAttributes: RedirectAttributes,
        model: Model,
        proj: String? = proj1
    ): String {
        if (bug.qcId == null) {
            val myUserDetails = SecurityContextHolder.getContext().authentication.name
            val userLogin = userRepository.findByUsername(myUserDetails)
            if (userLogin.isPresent) {
                bug.qcId = userLogin.get()
            }
        }
        println(bug)
        return if (bindingResult.hasErrors()) {
            model.addAttribute("user", user)
            model.addAttribute("platform", platform)
            model.addAttribute("qcResultType", qcResultType)
            "add_bug"
        } else {
            service.save(bug)
            redirectAttributes.addFlashAttribute("successMessage", "Success save bug")
            "redirect:/inPro"
        }
    }

    @RequestMapping(value = ["/save"], method = [RequestMethod.POST])
    fun saveBug(
        @Valid @ModelAttribute("bug") bug: ListBug,
        redirectAttributes: RedirectAttributes,
        result: BindingResult,
        request: HttpServletRequest
    ): String {
        val referer: String = request.getHeader("Referer")
        if (bug.qcId == null) {
            val myUserDetails = SecurityContextHolder.getContext().authentication.name
            val userLogin = userRepository.findByUsername(myUserDetails)
            if (userLogin.isPresent) {
                bug.qcId = userLogin.get()
            }
        }
        val successMessage: String = if (isValid(bug)) {
            service.save(bug)
            "Success save bug"
        } else {
            "Failed ! Please try again"
        }
        redirectAttributes.addFlashAttribute("successMessage", successMessage)
        return if (referer.endsWith("=index")) "redirect:/" else "redirect:/inPro"
    }

    @RequestMapping("/edit/{id}")
    fun showEditBugForm(@PathVariable(name = "id") id: Long, @RequestParam page: String): ModelAndView {
        val mav = ModelAndView("edit_bug")
        val bug: ListBug = service[id]
        val userQc: User = serUser[bug.qcId?.id]
        val userPro: User = serUser[bug.proId?.id]
        val plat: Platform = serPlatform[bug.platform!!.id]
        mav.addObject("listQc", userQc)
        mav.addObject("listPro", userPro)
        mav.addObject("platform", plat)
        mav.addObject("bug", bug)
        mav.addObject("worList", worList)
        mav.addObject("wStList", wStList)
        mav.addObject("dStList", dStList)
        mav.addObject("qcResultType", qcResultType)
        return mav
    }

    @RequestMapping("/delete/{id}")
    fun deleteBug(@PathVariable(name = "id") id: Long): String {
        service[id]
        service.delete(id)
        return "redirect:/"
    }

    @RequestMapping("/delete2/{id}")
    fun delete2Bug(@PathVariable(name = "id") id: Long): String {
        service[id]
        service.delete(id)
        return "redirect:/inPro"
    }

}

